package com.usky.demo.service.rabbitmq;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.shaded.com.google.gson.JsonObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.squareup.moshi.Json;
import com.usky.common.core.util.JsonUtils;
import com.usky.demo.domain.DataRealTime;
import com.usky.demo.mapper.SysUserMapper;
import com.usky.demo.service.config.CodeCache;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 中间处理消息转发
 */
@Service
@Repository
public class SimpleContext {
    @Autowired
    private SysUserMapper sysUserMapper;

    /**
     * 设备数据存mysql数据库
     * @param map
     */
    public void disposeMessageToDB(Map map){
        List<DataRealTime> list = new ArrayList<>();
        Map map_data = JsonUtils.fromJson(map.get("messageData").toString(),Map.class);
        String deviceId = map_data.get("device_id").toString();
        String tableName = "sp_d"+deviceId;

        //校验设备对应历史数据表在数据库中是否存在，不存在就创建表，并将这个表名添加到全局变量内存中
        if(!CodeCache.tablesList.contains(tableName)){
            if(sysUserMapper.existTable(tableName) == 0){
                sysUserMapper.createTable(tableName);
            }
            CodeCache.tablesList.add(tableName);
        }

        Object tt = JSONObject.toJSONString(map_data.get("metrics"));
        JSONObject metrics = JSON.parseObject(tt.toString());
        for(String entry : metrics.keySet()){
            DataRealTime dataRealTime = new DataRealTime();
            dataRealTime.setDeviceId(deviceId);
            dataRealTime.setProductCode(map_data.get("product_id").toString());
            //时间戳转localDateTime
            long timestamp = Long.valueOf(map_data.get("timestamp").toString()).longValue();
            LocalDateTime ltimestamp = LocalDateTime.ofEpochSecond(timestamp,0, ZoneOffset.ofHours(8));
            dataRealTime.setDataTime(ltimestamp);
            String deviceType = map_data.get("device_type").toString();
            String[] str = deviceType.split("-");
            dataRealTime.setDeviceType(Integer.parseInt(str[0]));
            dataRealTime.setInsertTime(LocalDateTime.now());
            dataRealTime.setAttributeName(entry.toLowerCase());
            dataRealTime.setAttributeData(metrics.get(entry).toString());
            list.add(dataRealTime);
        }
        sysUserMapper.DeviceRealDataToDB(list);
        sysUserMapper.DeviceDataToDB(list,deviceId);

    }
}














